@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: atoms.c,v 1.8 1999/02/25 00:33:02 nisse Exp $ */
+ * $Id: atoms.c,v 1.9 1999/09/06 22:08:01 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -25,7 +25,7 @@
 
 #include "atoms.h"
 
-#include <strings.h>
+#include <string.h>
 
 struct atom_rassoc
 {
