@@ -2,11 +2,11 @@
  *
  * user authorization database
  *
- * $Id: server_authorization.c,v 1.4 1999/11/07 00:08:13 nisse Exp $ */
+ * $Id: server_authorization.c,v 1.6 1999/11/16 22:13:40 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
- * Copyright (C) 1999 Balazs Scheidler
+ * Copyright (C) 1999 Balzs Scheidler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,6 +49,7 @@
 */
 
 static struct verifier *do_key_lookup(struct lookup_verifier *c,
+				      int method,
 				      struct lsh_string *keyholder,
 				      struct lsh_string *key)
 {
@@ -64,6 +65,9 @@ static struct verifier *do_key_lookup(struct lookup_verifier *c,
   if (!user)
     return NULL;
 
+  if (method != ATOM_SSH_DSS)
+    return NULL;
+  
   pubkey_spki = keyblob2spki(key);
   if (!pubkey_spki)
     return NULL;
