@@ -5,7 +5,7 @@
 
 /* lsh, an implementation of the ssh protocol
  *
- * Copyright (C) 1999 Balazs Scheidler
+ * Copyright (C) 1999 Balzs Scheidler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -100,7 +100,7 @@ do_authenticate(struct userauth *s,
 	  EXCEPTION_RAISE(e, &unsupported_publickey_algorithm);
 	  return;
 	}
-      v = LOOKUP_VERIFIER(lookup, username, keyblob);
+      v = LOOKUP_VERIFIER(lookup, algorithm, username, keyblob);
 
 #if DATAFELLOWS_WORKAROUNDS
       if ( v && (algorithm == ATOM_SSH_DSS)
