@@ -2,7 +2,7 @@
  *
  * System dependant password related functions.
  *
- * $Id: server_userauth.h,v 1.2 1999/09/22 01:22:31 nisse Exp $
+ * $Id: server_userauth.h,v 1.3 1999/11/07 00:12:23 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -36,6 +36,13 @@
 #include "server_userauth.h.x"
 #undef GABA_DECLARE
 
+/* FIXME: Could abstract some of this information into methods, for example
+ *
+ * o  verifying a password
+ * o  reading files in ~/.lsh
+ * o  cd:ing to the home directory
+ */
+
 /* GABA:
    (class
      (name unix_user)
@@ -49,7 +56,14 @@
        (passwd string)  ; Crypted passwd
        (home string)
        (shell string)))
-       
+*/
+
+/* GABA:
+   (class
+     (name user_db)
+     (vars
+       (lookup method "struct unix_user *"
+                      "struct lsh_string *name" "int free")))
 */
 
 struct unix_user *lookup_user(struct lsh_string *name, int free);
