@@ -2,7 +2,7 @@
  *
  * An implementation of Ron Rivest's S-expressions, used in spki.
  *
- * $Id: sexp.h,v 1.20 1999/10/11 21:02:40 nisse Exp $ */
+ * $Id: sexp.h,v 1.22 1999/11/16 21:41:42 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -30,6 +30,7 @@
 #include "command.h"
 #include "list.h"
 #include "lsh_argp.h"
+#include "parse.h"
 
 #include "sexp_table.h"
 
@@ -40,6 +41,9 @@ struct sexp_iterator;
 #include "sexp.h.x"
 #undef GABA_DECLARE
 
+/* FIXME: Do some constification. Strings inside sexpressions should
+ * always be const. */
+
 /* GABA:
    (class
      (name sexp)
@@ -71,17 +75,24 @@ struct sexp_iterator;
 
 
 /* Iterator abstraction idea taken from Ron's code */
+/* FIXME: It would make a lot of sense to use (meta)class methods for
+ * iterators. */
 /* GABA:
    (class
      (name sexp_iterator)
      (vars
        (get method "struct sexp *")
        (set method void "struct sexp *")
+       (assoc method "struct sexp *"
+              "UINT32 length" "const UINT8 *name"
+	      "struct sexp_iterator **i")
+       (left method unsigned)
        (next method void)))
 */
 
 #define SEXP_GET(i) ((i)->get((i)))
 #define SEXP_SET(i, v) ((i)->set((i), (v)))
+#define SEXP_ASSOC(s, l, n, i) ((s)->assoc((s), (l), (n), (i)))
 #define SEXP_NEXT(i) ((i)->next((i)))
 
 /* Syntax styles */
@@ -143,7 +154,7 @@ struct lsh_string *sexp2string(struct sexp *e);
 
 /* Returns an ATOM_FOO constant if e is a simple sexp string
  * corresponding to an atom. Or zero if that is not the case. */
-UINT32 sexp2atom(struct sexp *e);
+int sexp2atom(struct sexp *e);
 
 /* int sexp_null_cdr(struct sexp *e); */
 
@@ -155,27 +166,35 @@ int sexp_bignum_s(const struct sexp *e, mpz_t n);
 
 /* Utility functions for parsing spki objects. */
 
-/* FIXME: These function might get obsoleted by spki.c */
+int sexp_eq(struct sexp *e, UINT32 length, const UINT8 *name);
+struct sexp *sexp_assq(struct sexp_iterator *i, int atom);
 
+struct sexp_iterator *
+sexp_check_type(struct sexp *e, UINT32 length,
+		const UINT8 *name);
+
+#if 0
 int sexp_eqz(const struct sexp *e, const char *s);
 int sexp_check_type(struct sexp *e, const char *type,
 		    struct sexp_iterator **res);
 struct sexp *sexp_assz(struct sexp_iterator *i, const char *name);
-int sexp_get_un(struct sexp_iterator *i, const char *name, mpz_t n);
+#endif
+
+int sexp_get_un(struct sexp_iterator *i, int atom, mpz_t n);
 
 extern int sexp_char_classes[];
 
-/* Parsing sexp */
 
-/* GABA:
-   (class
-     (name sexp_handler)
-     (vars
-       ;; Called when a complete sexpression has been read.
-       (handler method int "struct sexp *e")))
-*/
+/* Simple sexp parser */
 
-#define HANDLE_SEXP(h, s) ((h)->handler((h), (s)))
+struct sexp *
+sexp_parse_canonical(struct simple_buffer *buffer);
+
+struct sexp *
+string_to_sexp(struct lsh_string *src, int free);
+
+
+/* Streamed parser */
 
 struct read_handler *
 make_read_sexp(int style, int goon,
